function derivtest = deriv_check(hyp, inffunc, meanfunc, covfunc, likfunc, X, Y)
% check derivatives using differencing
%
%    derivtest = deriv_check(hyp, inffunc, meanfunc, covfunc, likfunc, X, Y)
%
%    all inputs are similar to standard GPML functions
%
% Generalized Gaussian Process Models Toolbox (GPML Add-on)
% Copyright (c) by Antoni B. Chan, 2013-11-27


numchecks = 10;
adnlZ = {};
adnlZ_test = {};

delta = 0.00001;
dfields = {'cov', 'lik', 'mean'};

randstd = 2;

nparams(1) = eval(feval(covfunc{:}));
nparams(2) = eval(feval(likfunc{:}));
nparams(3) = eval(feval(meanfunc));

for n=1:numchecks
  testhyp = hyp;
  testhyp.cov = randstd*randn(size(testhyp.cov));
  if isfield(testhyp, 'lik') && ~isempty(testhyp.lik)
    testhyp.lik = randstd*randn(size(testhyp));
  else
    testhyp.lik = [];
  end
  if isfield(testhyp, 'mean') && ~isempty(testhyp.mean)
    testhyp.mean = randstd*randn(size(testhyp.mean));
  else
    testhyp.mean = [];   
  end
  
  % special cases
  switch(func2str(likfunc{1}))
   case 'likExBinomialFixedN'
    testhyp.lik = hyp.lik;  % reset N
  end
    
  
  fprintf('*** deriv_check: compute at point ***\n');
  % compute at point
  [nlZ, dnlZ] = gp(testhyp, inffunc, meanfunc, covfunc, likfunc, X, Y(:));
  
  % compute at delta point
  dnlZ_test = [];
  for jj=1:length(dfields)
    myfield = dfields{jj};
    tmpd    = [];
    if isfield(testhyp, myfield)
      for ii=1:length(getfield(testhyp, myfield))
	testhypd = testhyp;
	tmp = getfield(testhypd, myfield);
	tmp(ii) = tmp(ii) + delta;
	testhypd = setfield(testhypd, myfield, tmp);
	fprintf('+++ deriv_check: compute at offset (%d,%d) +++\n', jj, ii);
	[nlZd, dnlZd] = gp(testhypd, inffunc, meanfunc, covfunc, likfunc, X, Y(:));
	% derivative estimate by differencing
	tmpd(ii) = (nlZd - nlZ) / delta;
      end
    end
    dnlZ_test = setfield(dnlZ_test, myfield, tmpd(:));
  end

  adnlZ{n}      = dnlZ;
  adnlZ_test{n} = dnlZ_test;
  ahyp{n}       = testhyp;
end


% reshape for easier processing
all_dnlZ      = [];
all_dnlZ_test = [];
all_hyp       = [];
for jj=1:length(dfields)
  myfield = dfields{jj};
  tmp  = [];
  tmpt = [];
  tmph = [];
  for n=1:numchecks    
    foo  = getfield(adnlZ{n}, myfield);
    foot = getfield(adnlZ_test{n}, myfield);
    fooh  =getfield(ahyp{n}, myfield);
    tmp  = [tmp  foo(:) ];
    tmpt = [tmpt foot(:)];
    tmph = [tmph fooh(:)];
  end
  all_dnlZ      = setfield(all_dnlZ, myfield, tmp);
  all_dnlZ_test = setfield(all_dnlZ_test, myfield, tmpt);
  all_hyp       = setfield(all_hyp, myfield, tmph);
end

max_dnlZ  = [];
med_dnlZ  = [];
mean_dnlZ = [];
sign_dnlZ = [];

for jj=1:length(dfields)
  myfield = dfields{jj};
  
  tmp0 = getfield(all_hyp, myfield);
  tmp1 = getfield(all_dnlZ, myfield);
  tmp2 = getfield(all_dnlZ_test, myfield);

  tmpd = tmp1-tmp2;
  tmpp = tmpd./tmp1 * 100;
  tmps = sign(tmp1).*sign(tmp2);
  
  for ii=1:size(tmp1,1)
  
    [foo, fooi] = sort(tmp0(ii,:));
    
    for i=fooi(:)'
      % print results
      fprintf('%s(%d)= %9.4f; d=% 9.4f; dtest=% 9.4f; err= % 10.4g; %%=% 10.6f\n', ...
	      myfield, ii, tmp0(ii,i), tmp1(ii,i), tmp2(ii,i), ...
	      tmpd(ii, i), tmpp(ii, i));
    end
  end
  
  max_dnlZ  = setfield(max_dnlZ, myfield,  max(abs(tmpp),[],2));
  mean_dnlZ = setfield(mean_dnlZ, myfield, mean(abs(tmpp),2));
  med_dnlZ  = setfield(med_dnlZ, myfield,  median(abs(tmpp),2));
  sign_dnlZ = setfield(sign_dnlZ, myfield, sum(tmps<0,2));
end

sumstr = '';

% print summary
for jj=1:length(dfields)
  myfield = dfields{jj};
  tmpmax  = getfield(max_dnlZ, myfield);
  tmpmean = getfield(mean_dnlZ, myfield);
  tmpmed  = getfield(med_dnlZ, myfield);
  tmpsign = getfield(sign_dnlZ, myfield);
  
  if ~isempty(tmpmax)
    for ii=1:length(tmpmax);
      sumstr = [sumstr, ...
        sprintf('%s(%d): max%%=% 10.6f; mean%%=% 10.6f; med%%=% 10.6f; sign_changes=%d; \n', ...
        myfield, ii, tmpmax(ii), tmpmean(ii), tmpmed(ii), tmpsign(ii))];
    end
  end
end
disp(sumstr);

derivtest.max_dnlZ  = max_dnlZ;
derivtest.mean_dnlZ = mean_dnlZ;
derivtest.med_dnlZ  = med_dnlZ;
derivtest.sign_dnlZ = sign_dnlZ;
derivtest.sumstr = sumstr;
derivtest.all_dnlZ = all_dnlZ;
derivtest.all_dnlZ_test = all_dnlZ_test;
derivtest.all_hyp = all_hyp;

return


%%% OLD OUTPUT %%%  
for n=1:numchecks
  dnlZ      = adnlZ{n};
  dnlZ_test = adnlZ_test{n};
  for jj=1:length(dfields)
    myfield = dfields{jj};
    tmp1 = getfield(dnlZ, myfield)';
    tmp2 = getfield(dnlZ_test, myfield)';
    
    if ~isempty(tmp1)
      for ii=1:length(tmp1)
	% print results
	fprintf('d.%s(%d)=% 9.4f; test=% 9.4f; diff= % 10.4g; %%=% 10.6f\n', ...
		myfield, ii, tmp1(ii), tmp2(ii), ...
		tmp1(ii)-tmp2(ii), ...
		(tmp1(ii)-tmp2(ii))./tmp1(ii)*100);
      
	% get stats
      end                
    end    
  end
end
