function lf = likExBernoulli(hyp)
% Bernoulli likelihood function 
%   y in {0,1}
%
% Generalized Gaussian Process Models Toolbox (GPML Add-on)
% Copyright (c) by Antoni B. Chan, 2013-11-27

% TODO: calculate EP moments, etc

% return number of likelihood parameters
if nargin<1, lf = 0; return; end

%%% check/process y %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
lf.chky    = @(y) checkY(y);


%%% setup binomial (fixed N) %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
lf.a       = @() 1;
lf.da      = @() 0;
lf.b       = @(theta) log(1+exp(theta));
lf.db      = @(theta) exp(theta - log(1+exp(theta)));
lf.d2b     = @(theta) exp(theta - 2*log(1+exp(theta)));
lf.d3b     = @(theta) -1./(1+exp(theta)) ...
                      +3./((1+exp(theta)).^2) ...
                      -2./((1+exp(theta)).^3);
                  
lf.lh      = @(y) 0;
lf.dlh     = @(y) 0;
lf.theta   = @(eta) eta;
lf.dtheta  = @(eta) ones(size(eta));
lf.d2theta = @(eta) zeros(size(eta));
lf.d3theta = @(eta) zeros(size(eta));
lf.fix_phi = 1;
lf.etahat  = @(y) zeros(size(y));

%%% Taylor approximation %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% expansion point
lf.etahat  = @(y) zeros(size(y));

 function y = checkY(y)
oldy = y;
y = round(y);
if any(y ~= oldy)
% % %   N
% % %   oldy'
% % %   y'
% % %   error('y and N mismatch!');
% % %   keyboard
end