function lf = likExBinomial(hyp)
% Binomial likelihood function -- variable N
%   y in {0,..,N}/N
%
%   N   = round(exp(hyp)+1);
%   hyp = log(N-1)
%
% Generalized Gaussian Process Models Toolbox (GPML Add-on)
% Copyright (c) by Antoni B. Chan, 2013-11-27

% return number of likelihood parameters
if nargin<1, lf = 1; return; end


%%% get dispersion parameter %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
N = exp(hyp)+1;

% if predicting, round N
if nargin<5
  N = round(N);
end


%%% check/process y %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
lf.chky    = @(y) checkY(y);

%%% setup binomial (variable N) %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
lf.a       = @() 1/N;
lf.da      = @() -(N-1)/(N^2);
lf.b       = @(theta) log(1+exp(theta));
lf.db      = @(theta) exp(theta - log(1+exp(theta)));
lf.d2b     = @(theta) exp(theta - 2*log(1+exp(theta)));
lf.d3b     = @(theta) -1./(1+exp(theta)) ...
                      +3./((1+exp(theta)).^2) ...
                      -2./((1+exp(theta)).^3);
lf.lh      = @(y) gammaln(N+1) - gammaln(N*y+1) - gammaln(N*(1-y)+1);
lf.dlh     = @(y) N*(psi(N+1) - y.*psi(N*y+1) - (1-y).*psi(N*(1-y)+1));
lf.theta   = @(eta) eta;
lf.dtheta  = @(eta) ones(size(eta));
lf.d2theta = @(eta) zeros(size(eta));
lf.d3theta = @(eta) zeros(size(eta));
lf.fix_phi = 0;
lf.etahat  = @(y) zeros(size(y));

function ycheck = checkY(y)
myN = round(N);
oldy = y;
ycheck = round(myN*y)/myN;

if any(ycheck ~= oldy)
  N
  oldy'
  y'
  error('y and N mismatch!');
  keyboard
end


