function lf = likExBinomialFixedN(hyp)
% Binomial likelihood function -- fixed N
%   y in {0,..,N}/N
%
%   N   = round(exp(hyp)+1);
%   hyp = log(N-1)
%
% Generalized Gaussian Process Models Toolbox (GPML Add-on)
% Copyright (c) by Antoni B. Chan, 2013-11-27

% TODO: use logfactlut()

% return number of likelihood parameters
if nargin<1, lf = 1; return; end


%%% get dispersion parameter %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
N = round(exp(hyp)+1);


%%% check/process y %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
lf.chky    = @(y) checkY(N, y);


%%% setup binomial (fixed N) %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
lf.a       = @() 1/N;
lf.da      = @() -(N-1)/(N^2);
lf.b       = @(theta) log(1+exp(theta));
lf.db      = @(theta) exp(theta - log(1+exp(theta)));
lf.d2b     = @(theta) exp(theta - 2*log(1+exp(theta)));
lf.d3b     = @(theta) -1./(1+exp(theta)) ...
                      +3./((1+exp(theta)).^2) ...
                      -2./((1+exp(theta)).^3);
lf.lh      = @(y) gammaln(N+1) - gammaln(N*y+1) - gammaln(N*(1-y)+1);
lf.dlh     = @(y) N*(psi(N+1) - y.*psi(N*y+1) - (1-y).*psi(N*(1-y)+1));
lf.theta   = @(eta) eta;
lf.dtheta  = @(eta) ones(size(eta));
lf.d2theta = @(eta) zeros(size(eta));
lf.d3theta = @(eta) zeros(size(eta));
lf.fix_phi = 1;
lf.etahat  = @(y) zeros(size(y));

function y = checkY(N, y)
oldy = y;
y = round(N*y)/N;
if any(y ~= oldy) % removed for demo
% %   N
% %   oldy'
% %   y'
% %   error('y and N mismatch!');
% %   keyboard
end

