function lf = likExExponential(hyp)
% Exponential likelihood function
%  domain: y>=0 
%   y<0 is mapped to y=0
% 
%  p(y) = e^(-y/mu) / mu
%
%   mu is the mean
%   theta = -1/mu
%
% mapping is: mu = g^{-1}(eta) = e^eta
%           : theta(eta) = -e^-eta
%
% Note: this is equivalent to likExGamma with nu=1
%
% Generalized Gaussian Process Models Toolbox (GPML Add-on)
% Copyright (c) by Antoni B. Chan, 2013-11-27

% return number of likelihood parameters
if nargin<1, lf = 0; return; end

%%% get dispersion parameter %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% check/process y %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% move negative y to zero
lf.chky    = @(y) checkY(y);

%%% setup Gamma %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
lf.a       = @() 1;
lf.da      = @() 0;
lf.b       = @(theta) -log(-theta);
lf.db      = @(theta) -1./theta;
lf.d2b     = @(theta) 1./(theta.^2);
lf.d3b     = @(theta) -2./(theta.^3);
lf.lh      = @(y) 1;
lf.dlh     = @(y) 0;
lf.theta   = @(eta) -exp(-eta);
lf.dtheta  = @(eta) exp(-eta);
lf.d2theta = @(eta) -exp(-eta);
lf.d3theta = @(eta) exp(-eta);
lf.fix_phi = 1;


%%% Taylor approximation %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c = 0.001;
%lf.etahat  = @(y) log(y+c);
lf.etahat  = @(y) log(y);

function y = checkY(y)
ii = find(y<0);
if ~isempty(ii)
  y(ii) = 0;
  warning('y has negative values!');
end
