function lf = likExGauss(hyp)
% Gaussian likelihood function
%   y in real
%
%  hyp = log(sn) = 0.5log(sn2)
%
% Generalized Gaussian Process Models Toolbox (GPML Add-on)
% Copyright (c) by Antoni B. Chan, 2013-11-27


% return number of likelihood parameters
% if nargin<2, lf = {'1'}; return; end
if nargin<1, lf = 1; return; end


%%% check/process y %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
lf.chky    = @(y) y;

%%% get dispersion parameter %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
sn2 = exp(2*hyp);
% sn2 = exp(0.5 * hyp);

% added by lifeng
lf.nhyp = 1;

%%% setup Gaussian %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
lf.a       = @() sn2;
lf.da      = @() 2*sn2;
% lf.da      = @() 0.5*sn2;
lf.b       = @(theta) 0.5*(theta.*theta);
lf.db      = @(theta) theta;
lf.d2b     = @(theta) ones(size(theta));
lf.d3b     = @(theta) zeros(size(theta));
lf.lh      = @(y) -0.5*(y.*y)/sn2 - 0.5*log(2*pi*sn2);
lf.dlh     = @(y) (y.*y)/sn2 - 1;
% lf.dlh     = @(y) (y.*y)/sn2/4 - 1/4;
lf.theta   = @(eta) eta;
lf.dtheta  = @(eta) ones(size(eta));
lf.d2theta = @(eta) zeros(size(eta));
lf.d3theta = @(eta) zeros(size(eta));
lf.fix_phi = 0;

% Taylor options
lf.etahat  = @(y) zeros(size(y));

% EP moments -- custom functions (use anonymous function to capture sn2)
lf.EP_moments = @(y,mu,s2) EP_moments(sn2, y, mu, s2);  
% lf.EP_Eq      = 'numint';
lf.EP_Eq      = @(y,mu,s2) EP_Eq(sn2, y, mu, s2);

% for KL computation
lf.custom_KL = 1;  % flag of the custom derivative, 1 use custon derivative, else compute moments in generic fun
lf.dm        = @(m,v,y) (y-m)./sn2;
lf.d2m       = @(m,v,y) ones(size(m))*sn2;
lf.dv        = @(m,v,y) -ones(size(m))./2/sn2;
lf.d2v       = @(m,v,y) 0.5/v./sn2;
lf.dmv       = @(m,v,y) zeros(size(m));
lf.el        = @(m,v,y) -((y-mu).^2+s2)/(2*sn2)-log(2*pi*sn2)/2;
% expectation of theta and b
lf.etheta    = @(m,v) m;
lf.eb        = @(m,v) 0.5*(m.^2+v);


%%% EP FUNCTIONS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [lZ, qmu, qvar] = EP_moments(sn2, y, mu, s2)
ss = sn2+s2;

% lZ = log N(y|mu,sn2+s2)
%lZ = -0.5*(y-mu).^2./ss -0.5*log(2*pi*ss);

yn = size(y,2);
lZ = -0.5*(y-repmat(mu,[1,yn])).^2./repmat(ss,[1,yn]) - repmat(0.5*log(2*pi*ss), [1 yn]);

% q = N(y|eta,sn2)N(eta|mu,s2)/Z
if (nargout>1)
  % these can probably be simplified
  qmu = ((y-mu)./ss) .*s2 + mu;  
  if (nargout>2)
    qvar = (-1./ss) .*(s2.^2) + s2;
  end
end


function [Eq_th, Eq_bth] = EP_Eq(sn2, y, mu, s2)
ss = sn2+s2;
% these can probably be simplified
qmu  = ((y-mu)./ss) .*s2 + mu;
qvar = (-1./ss) .*(s2.^2) + s2;

% E_q[theta(eta)] = E_q[eta] = qmu
Eq_th = qmu;

% E_q[b(theta(eta))] = E_q[0.5 eta^2] = 0.5*(qvar+qmu^2)
Eq_bth = 0.5*(qvar + qmu.^2);
