function lf = likExGeometric(hyp)
% Geometric likelihood function 
%   y in {0,..,N}/N
%
%   N   = round(exp(hyp)+1);
%   hyp = log(N-1)
%
% Generalized Gaussian Process Models Toolbox (GPML Add-on)
% Copyright (c) by Antoni B. Chan, 2013-11-27

% return number of likelihood parameters
if nargin<1, lf = 0; return; end

%%% check y %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
lf.chky    = @(y) checkY(y);


%%% setup binomial (fixed N) %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
lf.a       = @() 1;
lf.da      = @() 0;
lf.b       = @(theta) -log(1-exp(theta));
lf.db      = @(theta) exp(theta - log(1-exp(theta)));
lf.d2b     = @(theta) exp(theta - 2*log(1-exp(theta)));
lf.d3b     = @(theta) 2*exp(2*theta - 3*log(1-exp(theta))) ...
                      + exp(theta-2*log(1-exp(theta)));
                  
lf.lh      = @(y) 0;
lf.dlh     = @(y) 0;
lf.theta   = @(eta) -log(1+exp(-eta));
lf.dtheta  = @(eta) 1./(1+exp(eta));
lf.d2theta = @(eta) -exp(eta - 2*log(1+exp(eta)));
lf.d3theta = @(eta) 1./(1+exp(eta)) - 3./((1+exp(eta)).^2) + 2./((1+exp(eta)).^3);
lf.fix_phi = 1;
lf.etahat  = @(y) zeros(size(y));

%%% Taylor approximation %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c = 0.001;
lf.etahat  = @(y) log(y+c);
%lf.etahat  = @(y) log(y);

function ycheck = checkY(y)
 ycheck = max(floor(y), 0);  % round and truncate
 if any(y ~= ycheck)
 % error('y contains non-counting numbers!'); % removed for demo experiments
 end

