function lf = likExNegBinomial2(hyp)
% Negative Binomial likelihood function
%   Y>0
%
%   phi = exp(hyp)
%   hyp = log(phi)
%
% Generalized Gaussian Process Models Toolbox (GPML Add-on)
% Copyright (c) by Antoni B. Chan, 2013-11-27


% TODO: use logfactlut()

% return number of likelihood parameters
if nargin<1, lf = 1; return; end


%%% get dispersion parameter %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
phi = exp(hyp);
% % phi = 0.4;
phi1 = 1 / phi;

%%% check/process y %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
lf.chky    = @(y) checkY(y);


%%% setup negative binomial %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
lf.a       = @() phi;
lf.da      = @() phi;
lf.b       = @(theta) -log(1-exp(theta*phi1-0.0001));
lf.db      = @(theta) phi1 * exp(theta * phi1 - log(1-exp(theta * phi1)));
lf.d2b     = @(theta) phi1^2 * exp(theta * phi1 - 2*log(1-exp(theta * phi1)));
lf.d3b     = @(theta) 2 * phi1^3 * exp(2*theta*phi1 - 3*log(1-exp(theta*phi1))) ...
                      + phi1^3 * exp(theta*phi1-2*log(1-exp(theta*phi1)));                  
                  
lf.lh      = @(y) gammaln(y+phi1) - gammaln(y+1) - gammaln(phi1);
lf.dlh     = @(y) phi1*(psi(0, phi1) - psi(0, y+phi1));
lf.theta   = @(eta) myTheta(eta); 
lf.dtheta  = @(eta) 1./(1+exp(eta));
lf.d2theta = @(eta) -exp(eta - 2*log(1+exp(eta)));
lf.d3theta = @(eta) 1./(1+exp(eta)) - 3./((1+exp(eta)).^2) + 2./((1+exp(eta)).^3);
lf.fix_phi = 0;

% special case: b has hyperparameter
lf.b_has_hyp  = 1;
lf.db_hyp     = @(theta) db_hyp(theta, phi);

% the ideal expansion point (as a function of loghyp)
lf.etahat   = @(y) myethat(y, phi); 
lf.etahat_has_hyp = 0;
lf.detahat_hyp    = @(y) detahat_hyp2(y, phi);

function r = myethat(y, phi)
phi = 0.3;
r = -log(((phi*y)./(1+phi*y)).^(-phi)-1);

function y = myTheta(eta)
    if eta > 100
       y = eta;
    else
       y = -log(1 + exp(-eta));
    end
    
function deta = detahat_hyp2(y, phi)
phi1 = 1/phi;
yphi = phi*y;
tmp  = log(1+yphi) + yphi./(1+yphi) - log(yphi) - 1;
tmp1 = 1./((y./(phi1+y)).^(-phi) - 1) + 1;
deta = -tmp.* tmp1 * phi;

function [db, dbp, dbpp] = db_hyp(theta, phi)
idxZero = find(theta == 0);
if ~isempty(idxZero)
    theta(idxZero) = theta(idxZero) + 0.001;
end
phi1 = 1/phi;
tphi = exp(theta*phi1);
tphi2 = exp(2*theta*phi1);

db = -phi1*theta.*tphi./(1-tphi);
if (nargout>1)
  %dbp = 1/phi - (theta/(phi^2)).*psi1;
  dbp = -phi1*tphi./(1-tphi)-phi1^2*theta.*tphi./(1-tphi).^2;
  if (nargout>2)
    %dbpp = -psi1/(phi^2) - theta.*psi2 / (phi^3);
    %dbpp = -2*phi1^2*tphi./(1-tphi).^2-phi1^3*(tphi-tphi2)./(1-tphi).^3;
    dbpp = -2*phi1^2*tphi./(1-tphi).^2-phi1^3*theta.*(tphi+tphi2)./(1-tphi).^3;
  end
end

function y = checkY(y)
 ii = find(y<=0);
 if ~isempty(ii)
   y(ii) = 0.0001;
   warning('y has negative values!');
 end