%%% derivatives of LL and d2LL and dLL wrt hyp %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Generalized Gaussian Process Models Toolbox (GPML Add-on)
% Copyright (c) by Antoni B. Chan, 2013-11-27
function [lp_h, d2lp_h, dlp_h] = dlogp_dhyp(lf, y, eta)
[lp, lpv, dlp, d2lp] = logp_and_d(lf, y, eta);
lp_h  = -(lf.da()/lf.a())*lpv + lf.dlh(y);

%dlogp = -(da_hyp/ahyp)*sum(v,1) + sum(dlh_hyp,1);
%dlogp = dlogp - sum(db_hyp,1)/ahyp;

% transform y?
if isfield(lf, 'T')
  Ty = lf.T(y);
else
  Ty = y;
end

if (nargout>1)
  % derivative of d2LL/deta2 wrt hyp
  %Xd2lp = ( (Ty-lf.db(te)).*d2te - lf.d2b(te).*(dte.^2) ) / lf.a();
  d2lp_h = -lf.da()/lf.a() * d2lp;
end

% derivative of dLL/deta wrt hyp
dlp_h = -lf.da()/lf.a() * dlp;

% add extra factors if b(theta,phi)
if isfield(lf, 'b_has_hyp') && lf.b_has_hyp
  te   = lf.theta(eta);
  dte  = lf.dtheta(eta);
  d2te = lf.d2theta(eta);
  [db_h, dbp_h, dbpp_h] = lf.db_hyp(te);
  
  lp_h   = lp_h  - db_h / lf.a();
  if (nargout>1)
    d2lp_h = d2lp_h - (dbpp_h.*(dte.^2) + d2te.*dbp_h) / lf.a();
  end
  
  dlp_h  = dlp_h - dte.*dbp_h/lf.a();
end


if 0
  %%% OLD CODE for old infLaplace
  % calculates dt_dwu, which is now replaced with dlp_h
  
  if (nargout>2)
    % dt - dw * u
    w = -1./d2lp;
    u = dlp;
    dw    = lf.da()/lf.a() * w;
    du    = -(lf.da()/lf.a()) * u;
    dt    = 0;
    
    if isfield(lf, 'b_has_hyp') && lf.b_has_hyp
      % add extra terms to derivatives when b(theta,phi)
      dw    = dw - (w.^2).*(dbpp_h.*dte.^2 + dbp_h.*d2te) / lf.a();
      du    = du - dte.*dbp_h / lf.a();
      dt    = w.*du + u.*dw;
      
      % avoid inf-inf
      dt_dwu = w.*du;
    else
      dt_dwu = dt - dw.*u;
    end
    %dt_dwu = dt - dw.*u;
    
    if any(isnan(dt_dwu))
      % 1st and 2nd derivative are both 0!
      warning('dt_dwu is NaN');
      ii = find(isnan(dt_dwu));
      dt_dwu(ii) = 0;
      %keyboard
    end
    if any(isinf(dt_dwu))
      warning('dt_dwu is inf');
      keyboard
    end
  end
end
