function [thall] = theta_logloss()
% theta log-loss function
%
%   theta(eta) = log(1+e^eta)
%
% i.e., the theta is a linear function of eta when eta>0, and
%        approaches zero when eta<0
%
% Generalized Gaussian Process Models Toolbox (GPML Add-on)
% Copyright (c) by Antoni B. Chan, 2013-11-27


thall.th   = @theta;
thall.dth  = @dtheta;
thall.d2th = @d2theta;
thall.d3th = @d3theta;



function th = theta(eta)
th = log(1+exp(eta));

% for large eta, log(1+exp(eta)) = eta
ii = find(eta>100);
th(ii) = eta(ii);


function dth = dtheta(eta)
dth = exp(eta - log(1+exp(eta)));
% check positive tail (limit is 1)
ii = find(eta>100);
dth(ii) = 1;


function d2th = d2theta(eta)
d2th = exp(eta - 2*log(1+exp(eta)));
% check positive tail (limit is 0)
ii = find(eta>100);
d2th(ii) = 0;


function d3th = d3theta(eta)
foo = log(1+exp(eta));
ii = find(eta>100);
foo(ii) = eta(ii);
t1 = 2*eta-3*foo;
t2 = eta-3*foo;
d3th = -exp(t1)+exp(t2);



